@@ -28,14 +28,15 @@ and user errors in SCons.
 
 """
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Errors.py 0.96.1.D001 2004/08/23 09:55:29 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Errors.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
 
 
 class BuildError(Exception):
-    def __init__(self, node=None, errstr="Unknown error", *args):
+    def __init__(self, node=None, errstr="Unknown error", filename=None, *args):
         self.node = node
         self.errstr = errstr
+        self.filename = filename
         apply(Exception.__init__, (self,) + args)
 
 class InternalError(Exception):
@@ -53,8 +54,9 @@ class ExplicitExit(Exception):
         self.status = status
         apply(Exception.__init__, (self,) + args)
 
-class ConfigureDryRunError(UserError):
-    """Raised when a file needs to be updated during a Configure process,
-    but the user requested a dry-run"""
-    def __init__(self,file):
-        UserError.__init__(self,"Cannot update configure test (%s) within a dry-run." % str(file))
+class TaskmasterException(Exception):
+    def __init__(self, node=None, exc_info=(None, None, None), *args):
+        self.node = node
+        self.errstr = "Exception"
+        self.exc_info = exc_info
+        apply(Exception.__init__, (self,) + args)
