@@ -8,7 +8,7 @@ This creates a hash of global Aliases (dummy targets).
 """
 
 #
-# Copyright (c) 2001, 2002 Steven Knight
+# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,8 +30,9 @@ This creates a hash of global Aliases (dummy targets).
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "src/engine/SCons/Node/Alias.py 0.D006 2002/03/28 02:47:47 __USER__"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Node/Alias.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
+import string
 import UserDict
 
 import SCons.Errors
@@ -39,19 +40,33 @@ import SCons.Node
 import SCons.Util
 
 class AliasNameSpace(UserDict.UserDict):
-    def Alias(self, name):
-        if self.has_key(name):
-            raise SCons.Errors.UserError
-        self[name] = SCons.Node.Alias.Alias(name)
-        return self[name]
+    def Alias(self, name, **kw):
+        if isinstance(name, SCons.Node.Alias.Alias):
+            return name
+        try:
+            a = self[name]
+        except KeyError:
+            a = apply(SCons.Node.Alias.Alias, (name,), kw)
+            self[name] = a
+        return a
 
-    def lookup(self, name):
+    def lookup(self, name, **kw):
         try:
             return self[name]
         except KeyError:
             return None
 
+class AliasNodeInfo(SCons.Node.NodeInfoBase):
+    pass
+
+class AliasBuildInfo(SCons.Node.BuildInfoBase):
+    pass
+
 class Alias(SCons.Node.Node):
+
+    NodeInfo = AliasNodeInfo
+    BuildInfo = AliasBuildInfo
+
     def __init__(self, name):
         SCons.Node.Node.__init__(self)
         self.name = name
@@ -59,35 +74,39 @@ class Alias(SCons.Node.Node):
     def __str__(self):
         return self.name
 
-    def build(self):
-        """A "builder" for aliases."""
-        pass
+    really_build = SCons.Node.Node.build
+    current = SCons.Node.Node.children_are_up_to_date
 
-    def set_bsig(self, bsig):
-        """An alias has no signature."""
-        pass
+    def is_under(self, dir):
+        # Make Alias nodes get built regardless of
+        # what directory scons was run from. Alias nodes
+        # are outside the filesystem:
+        return 1
 
-    def set_csig(self, csig):
-        """An alias has no signature."""
-        pass
-
-    def current(self):
-        """If all of our children were up-to-date, then this
-        Alias was up-to-date, too."""
-        state = 0
-        for kid in self.children(None):
-            s = kid.get_state()
-            if s and (not state or s > state):
-                state = s
-        if state == 0 or state == SCons.Node.up_to_date:
-            return 1
-        else:
-            return 0
+    def get_contents(self):
+        """The contents of an alias is the concatenation
+        of all the contents of its sources"""
+        contents = map(lambda n: n.get_contents(), self.children())
+        return string.join(contents, '')
 
     def sconsign(self):
         """An Alias is not recorded in .sconsign files"""
         pass
-        
+
+    #
+    #
+    #
+
+    def build(self):
+        """A "builder" for aliases."""
+        pass
+
+    def convert(self):
+        try: del self.builder
+        except AttributeError: pass
+        self.reset_executor()
+        self.build = self.really_build
+
 default_ans = AliasNameSpace()
 
 SCons.Node.arg2nodes_lookups.append(default_ans.lookup)
