@@ -31,12 +31,11 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/aixc++.py 0.96.1.D001 2004/08/23 09:55:29 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/aixc++.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
 import os.path
 
 import SCons.Platform.aix
-import SCons.Script.SConscript
 
 cplusplus = __import__('c++', globals(), locals(), [])
 
@@ -48,7 +47,7 @@ def get_xlc(env):
     return SCons.Platform.aix.get_xlc(env, xlc, xlc_r, packages)
 
 def smart_cxxflags(source, target, env, for_signature):
-    build_dir = SCons.Script.SConscript.GetBuildPath()
+    build_dir = env.GetBuildPath()
     if build_dir:
         return '-qtempinc=' + os.path.join(build_dir, 'tempinc')
     return ''
