@@ -27,7 +27,7 @@ Common DVI Builder definition for various other Tool modules that use it.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/dvi.py 0.96.92.D001 2006/04/10 23:13:27 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/dvi.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
 import SCons.Builder
 import SCons.Tool
@@ -47,7 +47,8 @@ def generate(env):
             DVIBuilder = SCons.Builder.Builder(action = {},
                                                source_scanner = SCons.Tool.LaTeXScanner,
                                                suffix = '.dvi',
-                                               emitter = {})
+                                               emitter = {},
+                                               source_ext_match = None)
 
         env['BUILDERS']['DVI'] = DVIBuilder
 
