@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/fortran.py 0.96.1.D001 2004/08/23 09:55:29 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/fortran.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
 import re
 import string
@@ -62,7 +62,7 @@ else:
 FortranScan = SCons.Scanner.Fortran.FortranScan("FORTRANPATH")
 
 for suffix in FortranSuffixes + FortranPPSuffixes:
-    SCons.Defaults.ObjSourceScan.add_scanner(suffix, FortranScan)
+    SCons.Tool.SourceFileScanner.add_scanner(suffix, FortranScan)
 
 #
 def _fortranEmitter(target, source, env):
@@ -78,9 +78,10 @@ def _fortranEmitter(target, source, env):
     modules = SCons.Util.unique(modules)
     # Convert module name to a .mod filename
     suffix = env.subst('$FORTRANMODSUFFIX')
+    moddir = env.subst('$FORTRANMODDIR')
     modules = map(lambda x, s=suffix: string.lower(x) + s, modules)
     for m in modules:
-       target.append(env.fs.File(m))
+       target.append(env.fs.File(m, moddir))
     return (target, source)
 
 def FortranEmitter(target, source, env):
@@ -94,7 +95,7 @@ def ShFortranEmitter(target, source, env):
 class VariableListGenerator:
     def __init__(self, *variablelist):
         self.variablelist = variablelist
-    def __call__(self, env, target, source, for_signature):
+    def __call__(self, env, target, source, for_signature=0):
         for v in self.variablelist:
             try: return env[v]
             except KeyError: pass
@@ -104,32 +105,40 @@ class VariableListGenerator:
 FortranGenerator = VariableListGenerator('FORTRAN', 'F77', '_FORTRAND')
 FortranFlagsGenerator = VariableListGenerator('FORTRANFLAGS', 'F77FLAGS')
 FortranCommandGenerator = VariableListGenerator('FORTRANCOM', 'F77COM', '_FORTRANCOMD')
+FortranCommandStrGenerator = VariableListGenerator('FORTRANCOMSTR', 'F77COMSTR', '_FORTRANCOMSTRD')
 FortranPPCommandGenerator = VariableListGenerator('FORTRANPPCOM', 'F77PPCOM', '_FORTRANPPCOMD')
+FortranPPCommandStrGenerator = VariableListGenerator('FORTRANPPCOMSTR', 'F77PPCOMSTR', '_FORTRANPPCOMSTRD')
 ShFortranGenerator = VariableListGenerator('SHFORTRAN', 'SHF77', 'FORTRAN', 'F77', '_FORTRAND')
 ShFortranFlagsGenerator = VariableListGenerator('SHFORTRANFLAGS', 'SHF77FLAGS')
 ShFortranCommandGenerator = VariableListGenerator('SHFORTRANCOM', 'SHF77COM', '_SHFORTRANCOMD')
+ShFortranCommandStrGenerator = VariableListGenerator('SHFORTRANCOMSTR', 'SHF77COMSTR', '_SHFORTRANCOMSTRD')
 ShFortranPPCommandGenerator = VariableListGenerator('SHFORTRANPPCOM', 'SHF77PPCOM', '_SHFORTRANPPCOMD')
+ShFortranPPCommandStrGenerator = VariableListGenerator('SHFORTRANPPCOMSTR', 'SHF77PPCOMSTR', '_SHFORTRANPPCOMSTRD')
 
 #
-FortranAction = SCons.Action.Action('$_FORTRANCOMG ')
-FortranPPAction = SCons.Action.Action('$_FORTRANPPCOMG ')
-ShFortranAction = SCons.Action.Action('$_SHFORTRANCOMG ')
-ShFortranPPAction = SCons.Action.Action('$_SHFORTRANPPCOMG ')
+FortranAction = SCons.Action.Action('$_FORTRANCOMG ', '$_FORTRANCOMSTRG')
+FortranPPAction = SCons.Action.Action('$_FORTRANPPCOMG ', '$_FORTRANPPCOMSTRG')
+ShFortranAction = SCons.Action.Action('$_SHFORTRANCOMG ', '$_SHFORTRANCOMSTRG')
+ShFortranPPAction = SCons.Action.Action('$_SHFORTRANPPCOMG ', '$_SHFORTRANPPCOMSTRG')
 
 def add_to_env(env):
     """Add Builders and construction variables for Fortran to an Environment."""
 
-    env['_FORTRANG']        = FortranGenerator
-    env['_FORTRANFLAGSG']   = FortranFlagsGenerator
-    env['_FORTRANCOMG']     = FortranCommandGenerator
-    env['_FORTRANPPCOMG']   = FortranPPCommandGenerator
+    env['_FORTRANG']            = FortranGenerator
+    env['_FORTRANFLAGSG']       = FortranFlagsGenerator
+    env['_FORTRANCOMG']         = FortranCommandGenerator
+    env['_FORTRANCOMSTRG']      = FortranCommandStrGenerator
+    env['_FORTRANPPCOMG']       = FortranPPCommandGenerator
+    env['_FORTRANPPCOMSTRG']    = FortranPPCommandStrGenerator
 
-    env['_SHFORTRANG']      = ShFortranGenerator
-    env['_SHFORTRANFLAGSG'] = ShFortranFlagsGenerator
-    env['_SHFORTRANCOMG']   = ShFortranCommandGenerator
-    env['_SHFORTRANPPCOMG'] = ShFortranPPCommandGenerator
+    env['_SHFORTRANG']          = ShFortranGenerator
+    env['_SHFORTRANFLAGSG']     = ShFortranFlagsGenerator
+    env['_SHFORTRANCOMG']       = ShFortranCommandGenerator
+    env['_SHFORTRANCOMSTRG']    = ShFortranCommandStrGenerator
+    env['_SHFORTRANPPCOMG']     = ShFortranPPCommandGenerator
+    env['_SHFORTRANPPCOMSTRG']  = ShFortranPPCommandStrGenerator
 
-    env['_FORTRANINCFLAGS'] = '$( ${_concat(INCPREFIX, FORTRANPATH, INCSUFFIX, __env__, RDirs)} $)'
+    env['_FORTRANINCFLAGS'] = '$( ${_concat(INCPREFIX, FORTRANPATH, INCSUFFIX, __env__, RDirs, TARGET, SOURCE)} $)'
 
     env['FORTRANMODPREFIX'] = ''     # like $LIBPREFIX
     env['FORTRANMODSUFFIX'] = '.mod' # like $LIBSUFFIX
@@ -137,7 +146,7 @@ def add_to_env(env):
     env['FORTRANMODDIR'] = ''          # where the compiler should place .mod files
     env['FORTRANMODDIRPREFIX'] = ''    # some prefix to $FORTRANMODDIR - similar to $INCPREFIX
     env['FORTRANMODDIRSUFFIX'] = ''    # some suffix to $FORTRANMODDIR - similar to $INCSUFFIX
-    env['_FORTRANMODFLAG'] = '$( ${_concat(FORTRANMODDIRPREFIX, FORTRANMODDIR, FORTRANMODDIRSUFFIX, __env__)} $)'
+    env['_FORTRANMODFLAG'] = '$( ${_concat(FORTRANMODDIRPREFIX, FORTRANMODDIR, FORTRANMODDIRSUFFIX, __env__, RDirs)} $)'
 
     env.AppendUnique(FORTRANSUFFIXES = FortranSuffixes + FortranPPSuffixes)
 
@@ -155,10 +164,10 @@ def add_to_env(env):
         static_obj.add_emitter(suffix, FortranEmitter)
         shared_obj.add_emitter(suffix, ShFortranEmitter)
 
-    env['_FORTRANCOMD']     = '$_FORTRANG $_FORTRANFLAGSG $_FORTRANINCFLAGS $_FORTRANMODFLAG -c -o $TARGET $SOURCES'
-    env['_FORTRANPPCOMD']   = '$_FORTRANG $_FORTRANFLAGSG $CPPFLAGS $_CPPDEFFLAGS $_FORTRANINCFLAGS $_FORTRANMODFLAG -c -o $TARGET $SOURCES'
-    env['_SHFORTRANCOMD']   = '$_SHFORTRANG $_SHFORTRANFLAGSG $_FORTRANINCFLAGS $_FORTRANMODFLAG -c -o $TARGET $SOURCES'
-    env['_SHFORTRANPPCOMD'] = '$_SHFORTRANG $_SHFORTRANFLAGSG $CPPFLAGS $_CPPDEFFLAGS $_FORTRANINCFLAGS $_FORTRANMODFLAG -c -o $TARGET $SOURCES'
+    env['_FORTRANCOMD']     = '$_FORTRANG -o $TARGET -c $_FORTRANFLAGSG $_FORTRANINCFLAGS $_FORTRANMODFLAG $SOURCES'
+    env['_FORTRANPPCOMD']   = '$_FORTRANG -o $TARGET -c $_FORTRANFLAGSG $CPPFLAGS $_CPPDEFFLAGS $_FORTRANINCFLAGS $_FORTRANMODFLAG $SOURCES'
+    env['_SHFORTRANCOMD']   = '$_SHFORTRANG -o $TARGET -c $_SHFORTRANFLAGSG $_FORTRANINCFLAGS $_FORTRANMODFLAG $SOURCES'
+    env['_SHFORTRANPPCOMD'] = '$_SHFORTRANG -o $TARGET -c $_SHFORTRANFLAGSG $CPPFLAGS $_CPPDEFFLAGS $_FORTRANINCFLAGS $_FORTRANMODFLAG $SOURCES'
 
 def generate(env):
     import f77
