@@ -32,7 +32,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/linkloc.py 0.96.1.D001 2004/08/23 09:55:29 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/linkloc.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
 import os.path
 import re
@@ -69,7 +69,7 @@ class LinklocGenerator:
         if for_signature:
             # Expand the contents of any linker command files recursively
             subs = 1
-            strsub = env.subst(self.cmdline)
+            strsub = env.subst(self.cmdline, target=target, source=source)
             while subs:
                 strsub, subs = _re_linker_command.subn(repl_linker_command, strsub)
             return strsub
@@ -94,7 +94,8 @@ def generate(env):
     env['LIBLINKPREFIX']='-lib '
     env['LIBLINKSUFFIX']='$LIBSUFFIX'
 
-    include_path, lib_path, exe_path = get_msvc_paths()
+    msvs_version = env.get('MSVS_VERSION')
+    include_path, lib_path, exe_path = get_msvc_paths(env, version = msvs_version)
     env['ENV']['LIB']            = lib_path
     env['ENV']['PATH']           = exe_path
 
