@@ -31,14 +31,16 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/m4.py 0.96.1.D001 2004/08/23 09:55:29 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/m4.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
+import SCons.Action
 import SCons.Builder
 import SCons.Util
 
 def generate(env):
     """Add Builders and construction variables for m4 to an Environment."""
-    bld = SCons.Builder.Builder(action = '$M4COM', src_suffix = '.m4')
+    M4Action = SCons.Action.Action('$M4COM', '$M4COMSTR')
+    bld = SCons.Builder.Builder(action = M4Action, src_suffix = '.m4')
 
     env['BUILDERS']['M4'] = bld
 
