@@ -31,8 +31,10 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/Tool/midl.py 0.96.1.D001 2004/08/23 09:55:29 knight"
+__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Tool/midl.py 0.96.93.D001 2006/11/06 08:31:54 knight"
 
+import SCons.Action
+import SCons.Builder
 import SCons.Defaults
 import SCons.Scanner.IDL
 import SCons.Util
@@ -53,11 +55,13 @@ def midl_emitter(target, source, env):
 
 idl_scanner = SCons.Scanner.IDL.IDLScan()
 
-midl_builder = SCons.Builder.Builder(action='$MIDLCOM',
+midl_action = SCons.Action.Action('$MIDLCOM', '$MIDLCOMSTR')
+
+midl_builder = SCons.Builder.Builder(action = midl_action,
                                      src_suffix = '.idl',
                                      suffix='.tlb',
                                      emitter = midl_emitter,
-                                     scanner = idl_scanner)
+                                     source_scanner = idl_scanner)
 
 def generate(env):
     """Add Builders and construction variables for midl to an Environment."""
@@ -73,5 +77,3 @@ def exists(env):
         return 1
     else:
         return env.Detect('midl')
-
-
